@@ -1,16 +1,16 @@
 from __future__ import (division)
 
 from pomegranate import *
+from pomegranate.bayes import BayesModel
 from pomegranate.io import DataGenerator
 from pomegranate.io import DataFrameGenerator
 
-from nose.tools import with_setup
-from nose.tools import assert_almost_equal
-from nose.tools import assert_equal
-from nose.tools import assert_not_equal
-from nose.tools import assert_less_equal
-from nose.tools import assert_raises
-from nose.tools import assert_true
+from .tools import with_setup
+from .tools import assert_equal
+from .tools import assert_not_equal
+from .tools import assert_true
+
+from numpy.testing import assert_almost_equal
 from numpy.testing import assert_array_almost_equal
 
 import pandas
@@ -136,6 +136,21 @@ def setup_multivariate():
 def teardown():
 	pass
 
+def test_unpickle_bayes_model():
+	"""Test that `BayesModel` can be pickled and unpickled."""
+	dists = [BernoulliDistribution(0.2), BernoulliDistribution(0.3)]
+	model = BayesModel(distributions=dists)
+	unpickled_model = pickle.loads(pickle.dumps(model))
+	np.testing.assert_almost_equal(model.weights, unpickled_model.weights)
+	# Check weights of individual distributions.
+	np.testing.assert_almost_equal(
+		model.distributions[0].parameters,
+		unpickled_model.distributions[0].parameters,
+	)
+	np.testing.assert_almost_equal(
+		model.distributions[1].parameters,
+		unpickled_model.distributions[1].parameters,
+	)
 
 @with_setup(setup_multivariate_gaussian, teardown)
 def test_bc_multivariate_gaussian_initialization():
@@ -560,7 +575,12 @@ def test_model():
 
 @with_setup(setup_hmm, teardown)
 def test_hmm_log_proba():
-	logs = model.predict_log_proba(np.array([list('H'), list('THHH'), list('TTTT'), list('THTHTHTHTHTH'), list('THTHHHHHTHTH')]))
+	logs = []
+	seqs = [list('H'), list('THHH'), list('TTTT'), list('THTHTHTHTHTH'), 
+		list('THTHHHHHTHTH')]
+
+	for seq in seqs:
+		logs.append(model.predict_log_proba(numpy.array([seq]))[0])
 
 	assert_almost_equal(logs[0][0], -0.89097292388986515)
 	assert_almost_equal(logs[0][1], -1.3609765531356006)
@@ -585,7 +605,12 @@ def test_hmm_log_proba():
 
 @with_setup(setup_hmm, teardown)
 def test_hmm_proba():
-	probs = model.predict_proba(np.array([list('H'), list('THHH'), list('TTTT'), list('THTHTHTHTHTH'), list('THTHHHHHTHTH')]))
+	probs = []
+	seqs = [list('H'), list('THHH'), list('TTTT'), list('THTHTHTHTHTH'), 
+		list('THTHHHHHTHTH')]
+
+	for seq in seqs:
+		probs.append(model.predict_proba(numpy.array([seq]))[0])
 
 	assert_almost_equal(probs[0][0], 0.41025641025641024)
 	assert_almost_equal(probs[0][1], 0.25641025641025639)
@@ -610,7 +635,12 @@ def test_hmm_proba():
 
 @with_setup(setup_hmm, teardown)
 def test_hmm_prediction():
-	predicts = model.predict(np.array([list('H'), list('THHH'), list('TTTT'), list('THTHTHTHTHTH'), list('THTHHHHHTHTH')]))
+	predicts = []
+	seqs = [list('H'), list('THHH'), list('TTTT'), list('THTHTHTHTHTH'), 
+		list('THTHHHHHTHTH')]
+
+	for seq in seqs:
+		predicts.append(model.predict(numpy.array([seq]))[0])
 
 	assert_equal(predicts[0], 0)
 	assert_equal(predicts[1], 0)
