@@ -18,13 +18,13 @@ from pomegranate import (Distribution,
 						 BernoulliDistribution,
 						 from_json)
 
-from nose.tools import with_setup
-from nose.tools import assert_almost_equal
-from nose.tools import assert_equal
-from nose.tools import assert_not_equal
-from nose.tools import assert_less_equal
-from nose.tools import assert_true
-from nose.tools import assert_raises
+from .tools import with_setup
+from .tools import assert_almost_equal
+from .tools import assert_equal
+from .tools import assert_not_equal
+from .tools import assert_less_equal
+from .tools import assert_true
+from .tools import assert_raises
 from numpy.testing import assert_array_equal
 from numpy.testing import assert_array_almost_equal
 import pickle
@@ -522,6 +522,10 @@ def test_distributions_discrete():
 	d = DiscreteDistribution.from_samples(['A', 'B', 'A', 'A'])
 	assert_equal(d.parameters[0], {'A': 0.75, 'B': 0.25})
 
+	# Test vector input instead of flat array.
+	d = DiscreteDistribution.from_samples(numpy.array(['A', 'B', 'A', 'A']).reshape(-1,1))
+	assert_equal(d.parameters[0], {'A': 0.75, 'B': 0.25})
+
 	d = DiscreteDistribution.from_samples(['A', 'B', 'A', 'A'], pseudocount=0.5)
 	assert_equal(d.parameters[0], {'A': 0.70, 'B': 0.30})
 
@@ -699,6 +703,12 @@ def test_distributions_poisson_random_sample():
 	assert_array_almost_equal(d.sample(5, random_state=5), x)
 	assert_raises(AssertionError, assert_array_almost_equal, d.sample(5), x)
 
+def test_beta():
+	"""Test pickling of beta distribution."""
+	d = BetaDistribution(2, 3)
+	e = pickle.loads(pickle.dumps(d))
+	assert_equal(e.name, "BetaDistribution")
+	assert_equal(e.parameters, [2, 3])
 
 def test_distributions_beta_random_sample():
 	d = BetaDistribution(1, 1)
