@@ -2,12 +2,12 @@ from pomegranate import *
 from pomegranate.io import DataGenerator
 from pomegranate.io import DataFrameGenerator
 
-from nose.tools import with_setup
-from nose.tools import assert_true
-from nose.tools import assert_equal
-from nose.tools import assert_greater
-from nose.tools import assert_raises
-from nose.tools import assert_not_equal
+from .tools import with_setup
+from .tools import assert_true
+from .tools import assert_equal
+from .tools import assert_greater
+from .tools import assert_raises
+from .tools import assert_not_equal
 from numpy.testing import assert_almost_equal
 from numpy.testing import assert_array_equal
 from numpy.testing import assert_array_almost_equal
@@ -90,6 +90,18 @@ def setup_multivariate_discrete():
 	gmm = GeneralMixtureModel([d1, d2], weights=np.array([0.4,0.6]))
 
 
+def setup_multivariate_mixed_discrete_other():
+	d1 = IndependentComponentsDistribution([PoissonDistribution(4.0),
+	                                        DiscreteDistribution({'A':0.5, 'B':0.5}), 
+	                                        DiscreteDistribution({'0':0.2, '1':0.2, '2':0.2, '3':0.4})])
+	d2 = IndependentComponentsDistribution([PoissonDistribution(1.0),
+	                                        DiscreteDistribution({'A':0.1, 'B':0.9}), 
+	                                        DiscreteDistribution({'0':0.1, '1':0.6, '2':0.1, '3':0.2})])
+
+
+	global gmm
+	gmm = GeneralMixtureModel([d1, d2], weights=np.array([0.4,0.6]))
+
 def teardown():
 	"""
 	Teardown the model, so delete it.
@@ -396,11 +408,13 @@ def test_gmm_multivariate_gaussian_fit_iterations():
 	numpy.random.seed(0)
 	X = numpy.concatenate([numpy.random.normal(i, 1, size=(100, 5)) for i in range(2)])
 
-	mu, cov = numpy.ones(5), numpy.eye(5)
-	d = [MultivariateGaussianDistribution(mu*i, cov) for i in range(2)]
-	gmm = GeneralMixtureModel(d)
-	gmm2 = gmm.copy()
-	gmm3 = gmm.copy()
+	gmms = []
+	for i in range(3):
+		mu, cov = numpy.ones(5), numpy.eye(5)
+		d = [MultivariateGaussianDistribution(mu*i, cov) for i in range(2)]
+		gmms.append(GeneralMixtureModel(d))
+	
+	gmm, gmm2, gmm3 = gmms
 
 	gmm.fit(X)
 	gmm2.fit(X, max_iterations=1)
@@ -414,28 +428,6 @@ def test_gmm_multivariate_gaussian_fit_iterations():
 	assert_equal(logp2, logp3)
 
 
-@with_setup(setup_multivariate_mixed, teardown)
-def test_gmm_multivariate_mixed_fit_iterations():
-	numpy.random.seed(0)
-	X = numpy.concatenate([numpy.random.normal(i, 1, size=(100, 5)) for i in range(2)])
-	X = numpy.abs(X)
-
-	gmm2 = gmm.copy()
-	gmm3 = gmm.copy()
-
-	gmm.fit(X)
-	gmm2.fit(X, max_iterations=1)
-	gmm3.fit(X, max_iterations=1)
-
-	logp1 = gmm.log_probability(X).sum()
-	logp2 = gmm2.log_probability(X).sum()
-	logp3 = gmm3.log_probability(X).sum()
-
-	assert_raises(AssertionError, assert_equal, logp1, logp2)
-	assert_equal(logp2, logp2)
-	assert_greater(logp1, logp2)
-
-
 def test_gmm_initialization():
 	assert_raises(ValueError, GeneralMixtureModel, [])
 
@@ -1232,6 +1224,22 @@ def test_gmm_multivariate_mixed_random_sample():
 	assert_array_almost_equal(gmm.sample(3, random_state=5), x)
 	assert_raises(AssertionError, assert_array_almost_equal, gmm.sample(3), x)
 
+
+@with_setup(setup_multivariate_mixed_discrete_other)
+def test_gmm_multivariate_mixed_discrete_other_fit():
+	x = numpy.array(
+		[
+			[0, 'A', '0'],
+			[5, 'A', '1'],
+			[4, 'A', '3'],
+			[5, 'B', '0']
+		], 
+		dtype=object
+	)
+	gmm.fit(x)
+	assert_array_almost_equal(gmm.weights, numpy.array([-0.28544206, -1.39304465]))
+
+
 def test_io_log_probability():
 	X = numpy.random.randn(100, 5) + 0.5
 	X2 = DataGenerator(X)
@@ -1330,4 +1338,4 @@ def test_io_from_samples_gmm():
 	logp1 = gmm1.log_probability(X)
 	logp2 = gmm2.log_probability(X)
 
-	assert_array_almost_equal(logp1, logp2)
\ No newline at end of file
+	assert_array_almost_equal(logp1, logp2)
