@@ -1,12 +1,12 @@
 from __future__ import (division)
 
 from pomegranate import *
-from nose.tools import with_setup
-from nose.tools import assert_almost_equal
-from nose.tools import assert_equal
-from nose.tools import assert_not_equal
-from nose.tools import assert_less_equal
-from nose.tools import assert_raises
+from .tools import with_setup
+from .tools import assert_almost_equal
+from .tools import assert_equal
+from .tools import assert_not_equal
+from .tools import assert_less_equal
+from .tools import assert_raises
 import random
 import pickle
 import numpy as np
